# ActivitySim
# See full license in LICENSE.txt.
import logging

import pandas as pd

from activitysim.core import tracing
from activitysim.core import config
from activitysim.core import pipeline
from activitysim.core import simulate
from activitysim.core import inject
from activitysim.core import expressions

from activitysim.abm.models.util import estimation

logger = logging.getLogger(__name__)


@inject.step()
def telecommute_frequency(persons_merged, persons, chunk_size, trace_hh_id):
    """
    This model predicts the frequency of telecommute for a person (worker) who
    does not works from home. The alternatives of this model are 'No Telecommute',
    '1 day per week', '2 to 3 days per week' and '4 days per week'. This model
    reflects the choices of people who prefer a combination of working from home and
    office during a week.

    The main interface to the telecommute frequency model is the telecommute_frequency() function.
    This function is registered as an orca step in the example Pipeline.
    """

    trace_label = "telecommute_frequency"
    model_settings_file_name = "telecommute_frequency.yaml"

    choosers = persons_merged.to_frame()
    choosers = choosers[choosers.workplace_zone_id > -1]

    logger.info("Running %s with %d persons", trace_label, len(choosers))

    model_settings = config.read_model_settings(model_settings_file_name)
    estimator = estimation.manager.begin_estimation("telecommute_frequency")

    constants = config.get_model_constants(model_settings)

    # - preprocessor
    preprocessor_settings = model_settings.get("preprocessor", None)
    if preprocessor_settings:

        locals_d = {}
        if constants is not None:
            locals_d.update(constants)

        expressions.assign_columns(
            df=choosers,
            model_settings=preprocessor_settings,
            locals_dict=locals_d,
            trace_label=trace_label,
        )

    model_spec = simulate.read_model_spec(file_name=model_settings["SPEC"])
    coefficients_df = simulate.read_model_coefficients(model_settings)
    model_spec = simulate.eval_coefficients(model_spec, coefficients_df, estimator)

    nest_spec = config.get_logit_model_settings(model_settings)

    if estimator:
        estimator.write_model_settings(model_settings, model_settings_file_name)
        estimator.write_spec(model_settings)
        estimator.write_coefficients(coefficients_df)
        estimator.write_choosers(choosers)

    choices = simulate.simple_simulate(
        choosers=choosers,
        spec=model_spec,
        nest_spec=nest_spec,
        locals_d=constants,
        chunk_size=chunk_size,
        trace_label=trace_label,
        trace_choice_name="telecommute_frequency",
        estimator=estimator,
    )

    choices = pd.Series(model_spec.columns[choices.values], index=choices.index)

    if estimator:
        estimator.write_choices(choices)
        choices = estimator.get_survey_values(
            choices, "persons", "telecommute_frequency"
        )
        estimator.write_override_choices(choices)
        estimator.end_estimation()

    persons = persons.to_frame()
    persons["telecommute_frequency"] = (
        choices.reindex(persons.index).fillna("").astype(str)
    )

    pipeline.replace_table("persons", persons)

    tracing.print_summary(
        "telecommute_frequency", persons.telecommute_frequency, value_counts=True
    )

    if trace_hh_id:
        tracing.trace_df(persons, label=trace_label, warn_if_empty=True)



# ActivitySim
# See full license in LICENSE.txt.

import logging
import sys

import pandas as pd

from activitysim.core import chunk, config, inject, mem, mp_tasks, pipeline, tracing

# from activitysim import abm


logger = logging.getLogger("activitysim")


def cleanup_output_files():

    active_log_files = [
        h.baseFilename
        for h in logger.root.handlers
        if isinstance(h, logging.FileHandler)
    ]
    tracing.delete_output_files("log", ignore=active_log_files)

    tracing.delete_output_files("h5")
    tracing.delete_output_files("csv")
    tracing.delete_output_files("txt")
    tracing.delete_output_files("yaml")
    tracing.delete_output_files("prof")


def run(run_list, injectables=None):

    if run_list["multiprocess"]:
        logger.info("run multiprocess simulation")
        mp_tasks.run_multiprocess(run_list, injectables)
    else:
        logger.info("run single process simulation")
        pipeline.run(models=run_list["models"], resume_after=run_list["resume_after"])
        pipeline.close_pipeline()
        mem.log_global_hwm()


def log_settings(injectables):

    settings = [
        "households_sample_size",
        "chunk_size",
        "multiprocess",
        "num_processes",
        "resume_after" "use_shadow_pricing",
        "hh_ids",
    ]

    for k in settings:
        logger.info("setting %s: %s" % (k, config.setting(k)))

    for k in injectables:
        logger.info("injectable %s: %s" % (k, inject.get_injectable(k)))


if __name__ == "__main__":

    # https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.set_option.html
    # pd.set_option('display.max_columns', 50)

    data_dir = "E:/projects/clients/ASIM/data/mtc_tm1"
    data_dir = "/Users/jeff.doyle/work/activitysim-data/mtc_tm1/data"
    data_dir = "../example/data"

    # inject.add_injectable('data_dir', '/Users/jeff.doyle/work/activitysim-data/mtc_tm1/data')
    inject.add_injectable("data_dir", ["ancillary_data", data_dir])
    # inject.add_injectable('data_dir', ['ancillary_data', '../activitysim/abm/test/data'])
    inject.add_injectable("configs_dir", ["configs", "../example/configs"])

    injectables = config.handle_standard_args()

    tracing.config_logger()
    config.filter_warnings()

    log_settings(injectables)

    t0 = tracing.print_elapsed_time()

    # cleanup if not resuming
    if not config.setting("resume_after", False):
        cleanup_output_files()

    run_list = mp_tasks.get_run_list()

    if run_list["multiprocess"]:
        # do this after config.handle_standard_args, as command line args may override injectables
        injectables = list(
            set(injectables) | set(["data_dir", "configs_dir", "output_dir"])
        )
        injectables = {k: inject.get_injectable(k) for k in injectables}
    else:
        injectables = None

    run(run_list, injectables)

    # pipeline.open_pipeline('_')
    #
    # households_df = pipeline.get_table('households')
    # print("households_df\n", households_df.head())
    #
    # tours_df = pipeline.get_table('tours')
    # print("tours_df\n", tours_df.head())
    #
    # pipeline.close_pipeline()

    t0 = tracing.print_elapsed_time("everything", t0)



# ActivitySim
# See full license in LICENSE.txt.

import os
import sys

import numpy as np
import openmatrix as omx
import pandas as pd

# currently hdf5 written with python3 works with both p2.7 and p3,
# but reading hdf5 built with p2.7 (tables==3.4.4) p3 throws a ValueError reading land_use_taz:
# ValueError: Buffer dtype mismatch, expected 'Python object' but got 'double'
if sys.version_info[0] < 3:
    raise Exception("Must be using Python 3")


# input files, SF county is zones 1 to 190, output files
source_store = "/Users/jeff.doyle/work/activitysim-data/mtc_tm1/data/mtc_asim.h5"
source_skims = "/Users/jeff.doyle/work/activitysim-data/mtc_tm1/data/skims.omx"

dest_data_dir = "/Users/jeff.doyle/work/activitysim-data"


def create_subset(dest_store, dest_skims, maxZone, households_sample_size=0):

    dest_store_path = os.path.join(dest_data_dir, dest_store)
    dest_skims_path = os.path.join(dest_data_dir, dest_skims)

    print("land_use_taz")
    df = pd.read_hdf(source_store, "land_use_taz")
    df = df[df.index <= maxZone]
    df.to_hdf(dest_store_path, "land_use_taz")
    del df

    print("households")
    hh_df = pd.read_hdf(source_store, "households")
    hh_df = hh_df[hh_df.TAZ <= maxZone]
    if households_sample_size:
        hh_df = hh_df.take(
            np.random.choice(len(hh_df), size=households_sample_size, replace=False)
        )
    hh_df.to_hdf(dest_store_path, "households")

    print("persons")
    per_df = pd.read_hdf(source_store, "persons")
    per_df = per_df[per_df.household_id.isin(hh_df.index)]
    per_df.to_hdf(dest_store_path, "persons")

    # process all skims
    skims = omx.open_file(source_skims)
    skims_out = omx.open_file(dest_skims_path, "w")

    skimsToProcess = skims.list_matrices()
    for skimName in skimsToProcess:
        print(skimName)
        skims_out[skimName] = skims[skimName][0:maxZone, 0:maxZone]
        skims_out[skimName].attrs.TITLE = ""  # remove funny character for OMX viewer


create_subset(
    dest_store="mtc_tm1_sf/data/mtc_asim.h5",
    dest_skims="mtc_tm1_sf/data/skims.omx",
    maxZone=190,
)

create_subset(
    dest_store="mtc_tm1_test/data/mtc_asim.h5",
    dest_skims="mtc_tm1_test/data/skims.omx",
    maxZone=25,
    households_sample_size=5000,
)



# ActivitySim
# See full license in LICENSE.txt.

__doc__ = "Activity-Based Travel Modeling"

try:
    from ._generated_version import __version__, __version_tuple__
except ImportError:
    # Package is not "installed", parse git tag at runtime
    from importlib.metadata import PackageNotFoundError, version

    try:
        __version__ = version(__package__)
    except PackageNotFoundError:
        # package is not installed
        __version__ = "999"

    __version_tuple__ = __version__.split(".")



# crop marin tvpb example data processing to one county
# Ben Stabler, ben.stabler@rsginc.com, 09/17/20

import os
import pandas as pd
import openmatrix as omx
import argparse
import numpy as np

MAZ_OFFSET = 0

MAZ_LIST = [
    1,
    2,
    7,
    8,
    9,
    10,
    11,
    12,
    13,
    14,
    15,
    16,
    17,
    18,
    19,
    20,
    21,
    22,
    23,
    24,
    25,
    26,
    27,
    28,
    29,
    30,
    31,
    32,
    33,
    34,
    35,
    36,
    37,
    38,
    39,
    40,
    41,
    42,
    43,
    44,
    45,
    46,
    47,
    49,
    50,
    53,
    54,
    55,
    56,
    57,
    58,
    59,
    60,
    181,
    182,
    184,
    185,
    186,
    187,
    188,
    190,
    191,
    192,
    193,
    194,
    195,
    196,
    197,
    198,
    199,
    200,
    201,
    202,
    203,
    204,
    205,
    206,
    207,
    208,
    209,
    278,
    279,
    280,
    281,
    282,
    283,
    284,
    285,
    286,
    287,
    288,
    289,
    290,
    291,
    292,
    298,
    365,
    370,
    371,
    372,
    373,
    374,
    375,
    376,
    377,
    378,
    379,
    380,
    381,
    383,
    384,
]


segments = {"test": {"TAZ": np.array(MAZ_LIST)}}

parser = argparse.ArgumentParser(description="crop raw_data")
parser.add_argument(
    "segment_name",
    metavar="segment_name",
    type=str,
    nargs=1,
    help=f"geography segmentation (e.g. full)",
)

parser.add_argument(
    "-c",
    "--check_geography",
    default=False,
    action="store_true",
    help="check consistency of MAZ, TAZ, TAP zone_ids and foreign keys & write orphan_households file",
)

args = parser.parse_args()


segment_name = args.segment_name[0]
check_geography = args.check_geography

assert segment_name in segments.keys(), f"Unknown seg: {segment_name}"

input_dir = "./data_raw"
output_dir = f"./data_{segment_name}"


print(f"segment_name {segment_name}")

print(f"input_dir {input_dir}")
print(f"output_dir {output_dir}")

print(f"check_geography {check_geography}")

if not os.path.isdir(output_dir):
    print(f"creating output directory {output_dir}")
    os.mkdir(output_dir)


def input_path(file_name):
    return os.path.join(input_dir, file_name)


def output_path(file_name):
    return os.path.join(output_dir, file_name)


def patch_maz(df, maz_offset):
    for c in df.columns:
        if c in ["maz", "OMAZ", "DMAZ", "mgra", "orig_mgra", "dest_mgra"]:
            df[c] += maz_offset
    return df


def read_csv(file_name):
    df = pd.read_csv(input_path(file_name))
    print(f"read {file_name} {df.shape}")
    return df


def to_csv(df, file_name):
    df.to_csv(output_path(file_name), index=False)
    print(f"write {file_name} {df.shape}")


def crop_omx(omx_file_name, zones, num_outfiles=1):
    skim_data_type = np.float32
    omx_in = omx.open_file(input_path(f"{omx_file_name}.omx"))
    print(f"omx_in shape {omx_in.shape()}")

    offset_map = None
    for mapping_name in omx_in.listMappings():
        _offset_map = np.asanyarray(omx_in.mapentries(mapping_name))
        if (
            offset_map is not None
            or not (_offset_map == np.arange(1, len(_offset_map) + 1)).all()
        ):
            assert offset_map is None or (offset_map == _offset_map).all()
            offset_map = _offset_map

    if offset_map is not None:
        om = pd.Series(offset_map)
        om = om[om.isin(zones.values)]
        nm = om[~om.isin(zones.values)]
        indexes = om.index.values

    else:
        indexes = zones.index.tolist()  # index of TAZ in skim (zero-based, no mapping)
    labels = zones.values  # TAZ zone_ids in omx index order

    # create
    if num_outfiles == 1:
        omx_out = [omx.open_file(output_path(f"{omx_file_name}.omx"), "w")]
    else:
        omx_out = [
            omx.open_file(output_path(f"{omx_file_name}{i + 1}.omx"), "w")
            for i in range(num_outfiles)
        ]

    for omx_file in omx_out:
        omx_file.create_mapping("zone_number", labels)

    iskim = 0
    for mat_name in omx_in.list_matrices():
        # make sure we have a vanilla numpy array, not a CArray
        m = np.asanyarray(omx_in[mat_name]).astype(skim_data_type)
        m = m[indexes, :][:, indexes]
        print(f"{mat_name} {m.shape}")

        omx_file = omx_out[iskim % num_outfiles]
        omx_file[mat_name] = m
        iskim += 1

    omx_in.close()

    for omx_file in omx_out:
        omx_file.close()


# non-standard input file names

LAND_USE = "land_use.csv"
HOUSEHOLDS = "households.csv"
PERSONS = "persons.csv"
MAZ_TAZ = "maz.csv"
TAP_MAZ = "tap.csv"
TAZ = "taz.csv"
SUBZONE = "subzoneData.csv"


if check_geography:

    # ######## check for orphan_households not in any maz in land_use
    land_use = read_csv(LAND_USE)
    land_use = land_use[["maz", "taz"]]
    land_use = land_use.sort_values(["taz", "maz"])

    households = read_csv(HOUSEHOLDS)
    orphan_households = households[~households.maz.isin(land_use.maz)]
    print(f"{len(orphan_households)} orphan_households")

    # write orphan_households to INPUT directory (since it doesn't belong in output)
    if len(orphan_households) > 0:
        file_name = "orphan_households.csv"
        print(
            f"writing {file_name} {orphan_households.shape} to {input_path(file_name)}"
        )
        orphan_households.to_csv(input_path(file_name), index=False)

    # ######## check that land_use and maz and taz tables have same MAZs and TAZs

    # could just build maz and taz files, but want to make sure PSRC data is right

    land_use = read_csv(LAND_USE)
    # assert land_use.set_index('MAZ').index.is_monotonic_increasing

    land_use = land_use.sort_values("maz")
    maz = read_csv(MAZ_TAZ).sort_values("MAZ")

    # ### FATAL ###
    if not land_use.maz.isin(maz.MAZ).all():
        print(
            f"land_use.MAZ not in maz.MAZ\n{land_use.maz[~land_use.maz.isin(maz.maz)]}"
        )
        # raise RuntimeError(f"land_use.MAZ not in maz.MAZ")

    if not maz.MAZ.isin(land_use.maz).all():
        print(f"maz.MAZ not in land_use.MAZ\n{maz.maz[~maz.maz.isin(land_use.maz)]}")

    # ### FATAL ###
    if not land_use.taz.isin(maz.TAZ).all():
        print(
            f"land_use.TAZ not in maz.TAZ\n{land_use.taz[~land_use.taz.isin(maz.taz)]}"
        )
        raise RuntimeError(f"land_use.TAZ not in maz.TAZ")

    if not maz.TAZ.isin(land_use.taz).all():
        print(f"maz.TAZ not in land_use.TAZ\n{maz.taz[~maz.taz.isin(land_use.taz)]}")


# land_use
land_use = read_csv(LAND_USE)
ur_land_use = land_use.copy()

slicer = segments[segment_name]
print(land_use.columns)
for slice_col, slice_values in slicer.items():
    # print(f"slice {slice_col}: {slice_values}")
    land_use = land_use[land_use[slice_col].isin(slice_values)]

print(f"land_use shape after slicing {land_use.shape}")
to_csv(land_use, "land_use.csv")


# TAZ
taz = pd.DataFrame({"TAZ": sorted(ur_land_use.TAZ.unique())})
taz = taz[taz.TAZ.isin(land_use["TAZ"])]
to_csv(taz, TAZ)

# maz_taz
if os.path.exists(MAZ_TAZ):
    maz_taz = read_csv(MAZ_TAZ).sort_values("MAZ")
    maz_taz = maz_taz[maz_taz.MAZ.isin(land_use.maz)]
    to_csv(maz_taz, MAZ_TAZ)

# tap
if os.path.exists(TAP_MAZ):
    taps = read_csv(TAP_MAZ)
    taps = taps[["TAP", "MAZ"]].sort_values(by="TAP").reset_index(drop=True)
    taps = taps[taps["MAZ"].isin(land_use["maz"])]
    to_csv(taps, "tap.csv")

# maz to tap
if os.path.exists("maz_to_tap_walk.csv"):
    maz_tap_walk = read_csv("maz_to_tap_walk.csv").sort_values(["MAZ", "TAP"])
    maz_tap_walk = maz_tap_walk[
        maz_tap_walk["MAZ"].isin(land_use["maz"])
        & maz_tap_walk["TAP"].isin(taps["TAP"])
    ]
    to_csv(maz_tap_walk, "maz_to_tap_walk.csv")
if os.path.exists("maz_to_tap_drive.csv"):
    taz_tap_drive = read_csv("maz_to_tap_drive.csv").sort_values(["MAZ", "TAP"])
    taz_tap_drive = taz_tap_drive[
        taz_tap_drive["MAZ"].isin(land_use["maz"])
        & taz_tap_drive["TAP"].isin(taps["TAP"])
    ]
    to_csv(taz_tap_drive, "maz_to_tap_drive.csv")

# maz to maz
if os.path.exists("maz_to_maz_walk.csv"):
    maz_maz_walk = read_csv("maz_to_maz_walk.csv").sort_values(["OMAZ", "DMAZ"])
    maz_maz_walk = maz_maz_walk[
        maz_maz_walk["OMAZ"].isin(land_use["maz"])
        & maz_maz_walk["DMAZ"].isin(land_use["maz"])
    ]
    to_csv(maz_maz_walk, "maz_to_maz_walk.csv")

if os.path.exists("maz_to_maz_bike.csv"):
    maz_maz_bike = read_csv("maz_to_maz_bike.csv").sort_values(["OMAZ", "DMAZ"])
    maz_maz_bike = maz_maz_bike[
        maz_maz_bike["OMAZ"].isin(land_use["maz"])
        & maz_maz_bike["DMAZ"].isin(land_use["maz"])
    ]
    to_csv(maz_maz_bike, "maz_to_maz_bike.csv")

# tap_lines
if os.path.exists("tapLines.csv"):
    tap_lines = read_csv("tapLines.csv")
    tap_lines = tap_lines[tap_lines["TAP"].isin(taps["TAP"])]
    to_csv(tap_lines, "tapLines.csv")

# households
households = read_csv(HOUSEHOLDS)
households = households[households["TAZ"].isin(land_use["TAZ"])]
to_csv(households, "households.csv")

# persons
persons = read_csv(PERSONS)
persons = persons[persons["household_id"].isin(households["household_id"])]
to_csv(persons, "persons.csv")

# Subzone (added ASR)
if os.path.exists(SUBZONE):
    subzone = read_csv(SUBZONE)
    subzone = subzone[subzone["subzone09"].isin(land_use["maz"])]
    to_csv(subzone, "subzone.csv")

# skims
crop_omx("taz_skims", taz.TAZ, num_outfiles=(4 if segment_name == "full" else 1))



# This file allows running ActivitySim as a Python module from the command line.
# For example:
#
#   python -m activitysim run ...
#
# This style of calling ActivitySim permits developers to more easily ensure
# they are calling the correct version that has been installed with the
# particular Python executable invoked, especially for debugging.  It also can
# be configured to invoke differ configurations automatically (like engaging the
# threadstopper options to prevent multithread thrashing). It is probably not
# needed by typical users with only one installed version.

import os
import sys


def main():
    # clean up message formatting
    if sys.argv and sys.argv[0].endswith("__main__.py"):
        sys.argv[0] = "activitysim"

    # threadstopper
    if "--fast" not in sys.argv:
        os.environ["MKL_NUM_THREADS"] = "1"
        os.environ["OMP_NUM_THREADS"] = "1"
        os.environ["OPENBLAS_NUM_THREADS"] = "1"
        os.environ["NUMBA_NUM_THREADS"] = "1"
        os.environ["VECLIB_MAXIMUM_THREADS"] = "1"
        os.environ["NUMEXPR_NUM_THREADS"] = "1"

    from .cli.main import main

    main()


if __name__ == "__main__":
    main()



import argparse
import os
import pathlib

GITHUB_REPOSITORY_OWNER = os.environ.get(
    "GITHUB_REPOSITORY_OWNER", "ActivitySim"
).lower()

parser = argparse.ArgumentParser()
parser.add_argument("path", type=pathlib.Path)
args = parser.parse_args()

if GITHUB_REPOSITORY_OWNER != "activitysim":
    content = args.path.read_text()
    new_content = content.replace(
        "activitysim.github.io", f"{GITHUB_REPOSITORY_OWNER}.github.io"
    )
    args.path.write_text(new_content)



#############################################################
# ActivitySim verification against TM1
# Ben Stabler, ben.stabler@rsginc.com, 02/22/19
# C:\projects\activitysim\verification>python compare_results.py
#############################################################

import openmatrix as omx
import pandas as pd

#############################################################
# INPUTS
#############################################################

pipeline_filename = "asim/pipeline.h5"
distance_matrix_filename = "asim/skims.omx"
asim_nmtf_alts_filename = "asim/non_mandatory_tour_frequency_alternatives.csv"

process_sp = True  # False skip work/sch shadow pricing comparisons, True do them
process_tm1 = True  # False only processes asim, True processes tm1 as well

asim_sp_work_filename = "asim/shadow_price_workplace_modeled_size_10.csv"
asim_sp_school_filename = "asim/shadow_price_school_modeled_size_10.csv"
asim_sp_school_no_sp_filename = "asim/shadow_price_school_modeled_size_1.csv"

tm1_access_filename = "tm1/accessibility.csv"
tm1_sp_filename = "tm1/ShadowPricing_9.csv"
tm1_work_filename = "tm1/wsLocResults_1.csv"
tm1_ao_filename = "tm1/aoResults.csv"
tm1_hh_filename = "tm1/householdData_1.csv"
tm1_cdap_filename = "tm1/cdapResults.csv"
tm1_per_filename = "tm1/personData_1.csv"
tm1_tour_filename = "tm1/indivTourData_1.csv"
tm1_jtour_filename = "tm1/jointTourData_1.csv"
tm1_trips_filename = "tm1/indivTripData_1.csv"
tm1_jtrips_filename = "tm1/jointTripData_1.csv"

#############################################################
# OUTPUT FILES FOR DEBUGGING
#############################################################

asim_zones_filename = "asim/asim_zones.csv"
asim_access_filename = "asim/asim_access.csv"
asim_per_filename = "asim/asim_per.csv"
asim_hh_filename = "asim/asim_hh.csv"
asim_tour_filename = "asim/asim_tours.csv"
asim_trips_filename = "asim/asim_trips.csv"

#############################################################
# COMMON LABELS
#############################################################

ptypes = [
    "",
    "Full-time worker",
    "Part-time worker",
    "University student",
    "Non-worker",
    "Retired",
    "Student of driving age",
    "Student of non-driving age",
    "Child too young for school",
]

mode_labels = [
    "",
    "DRIVEALONEFREE",
    "DRIVEALONEPAY",
    "SHARED2FREE",
    "SHARED2PAY",
    "SHARED3FREE",
    "SHARED3PAY",
    "WALK",
    "BIKE",
    "WALK_LOC",
    "WALK_LRF",
    "WALK_EXP",
    "WALK_HVY",
    "WALK_COM",
    "DRIVE_LOC",
    "DRIVE_LRF",
    "DRIVE_EXP",
    "DRIVE_HVY",
    "DRIVE_COM",
]

#############################################################
# DISTANCE SKIM
#############################################################

# read distance matrix (DIST)
distmat = omx.open_file(distance_matrix_filename)["DIST"][:]

#############################################################
# EXPORT TABLES
#############################################################

# write tables for verification
tazs = pd.read_hdf(pipeline_filename, "land_use/initialize_landuse")
tazs["zone"] = tazs.index
tazs.to_csv(asim_zones_filename, index=False)

access = pd.read_hdf(pipeline_filename, "accessibility/compute_accessibility")
access.to_csv(asim_access_filename, index=False)

hh = pd.read_hdf(pipeline_filename, "households/joint_tour_frequency")
hh["household_id"] = hh.index
hh.to_csv(asim_hh_filename, index=False)

per = pd.read_hdf(pipeline_filename, "persons/non_mandatory_tour_frequency")
per["person_id"] = per.index
per.to_csv(asim_per_filename, index=False)

tours = pd.read_hdf(pipeline_filename, "tours/stop_frequency")
tours["tour_id"] = tours.index
tours.to_csv(asim_tour_filename, index=False)

trips = pd.read_hdf(pipeline_filename, "trips/trip_mode_choice")
trips["trip_id"] = trips.index
trips.to_csv(asim_trips_filename, index=False)

#############################################################
# AGGREGATE
#############################################################

# accessibilities

if process_tm1:
    tm1_access = pd.read_csv(tm1_access_filename)
    tm1_access.to_csv("outputs/tm1_access.csv", na_rep=0)

asim_access = pd.read_csv(asim_access_filename)
asim_access.to_csv("outputs/asim_access.csv", na_rep=0)

#############################################################
# HOUSEHOLD AND PERSON
#############################################################

# work and school location

if process_sp:

    if process_tm1:
        tm1_markets = [
            "work_low",
            "work_med",
            "work_high",
            "work_high",
            "work_very high",
            "university",
            "school_high",
            "school_grade",
        ]
        tm1 = pd.read_csv(tm1_sp_filename)
        tm1 = tm1.groupby(tm1["zone"]).sum()
        tm1["zone"] = tm1.index
        tm1 = tm1.loc[tm1["zone"] > 0]
        ws_size = tm1[["zone"]]
        for i in range(len(tm1_markets)):
            ws_size[tm1_markets[i] + "_modeledDests"] = tm1[
                tm1_markets[i] + "_modeledDests"
            ]
        ws_size.to_csv("outputs/tm1_work_school_location.csv", na_rep=0)

    asim_markets = [
        "work_low",
        "work_med",
        "work_high",
        "work_high",
        "work_veryhigh",
        "university",
        "highschool",
        "gradeschool",
    ]
    asim = pd.read_csv(asim_sp_work_filename)
    asim_sch = pd.read_csv(asim_sp_school_filename)
    asim_sch_no_sp = pd.read_csv(asim_sp_school_no_sp_filename)
    asim_sch["gradeschool"] = asim_sch_no_sp[
        "gradeschool"
    ]  # grade school not shadow priced
    asim = asim.set_index("TAZ", drop=False)
    asim_sch = asim_sch.set_index("TAZ", drop=False)

    asim["gradeschool"] = asim_sch["gradeschool"].loc[asim["TAZ"]].tolist()
    asim["highschool"] = asim_sch["highschool"].loc[asim["TAZ"]].tolist()
    asim["university"] = asim_sch["university"].loc[asim["TAZ"]].tolist()

    ws_size = asim[["TAZ"]]
    for i in range(len(asim_markets)):
        ws_size[asim_markets[i] + "_asim"] = asim[asim_markets[i]]
    ws_size.to_csv("outputs/asim_work_school_location.csv", na_rep=0)

    # work county to county flows
    tazs = pd.read_csv(asim_zones_filename)
    counties = ["", "SF", "SM", "SC", "ALA", "CC", "SOL", "NAP", "SON", "MAR"]
    tazs["COUNTYNAME"] = pd.Series(counties)[tazs["county_id"].tolist()].tolist()
    tazs = tazs.set_index("zone", drop=False)

    if process_tm1:
        tm1_work = pd.read_csv(tm1_work_filename)
        tm1_work["HomeCounty"] = tazs["COUNTYNAME"].loc[tm1_work["HomeTAZ"]].tolist()
        tm1_work["WorkCounty"] = (
            tazs["COUNTYNAME"].loc[tm1_work["WorkLocation"]].tolist()
        )
        tm1_work_counties = tm1_work.groupby(["HomeCounty", "WorkCounty"]).count()[
            "HHID"
        ]
        tm1_work_counties = tm1_work_counties.reset_index()
        tm1_work_counties = tm1_work_counties.pivot(
            index="HomeCounty", columns="WorkCounty"
        )
        tm1_work_counties.to_csv("outputs/tm1_work_counties.csv", na_rep=0)

    asim_cdap = pd.read_csv(asim_per_filename)
    asim_cdap["HomeCounty"] = tazs["COUNTYNAME"].loc[asim_cdap["home_taz"]].tolist()
    asim_cdap["WorkCounty"] = (
        tazs["COUNTYNAME"].loc[asim_cdap["workplace_zone_id"]].tolist()
    )
    asim_work_counties = asim_cdap.groupby(["HomeCounty", "WorkCounty"]).count()[
        "household_id"
    ]
    asim_work_counties = asim_work_counties.reset_index()
    asim_work_counties = asim_work_counties.pivot(
        index="HomeCounty", columns="WorkCounty"
    )
    asim_work_counties.to_csv("outputs/asim_work_counties.csv", na_rep=0)

# auto ownership - count of hhs by num autos by taz

if process_tm1:
    tm1_ao = pd.read_csv(tm1_ao_filename)
    tm1_hh = pd.read_csv(tm1_hh_filename)
    tm1_ao = tm1_ao.set_index("HHID", drop=False)
    tm1_hh["ao"] = tm1_ao["AO"].loc[tm1_hh["hh_id"]].tolist()
    tm1_autos = tm1_hh.groupby(["taz", "ao"]).count()["hh_id"]
    tm1_autos = tm1_autos.reset_index()
    tm1_autos = tm1_autos.pivot(index="taz", columns="ao")
    tm1_autos.to_csv("outputs/tm1_autos.csv", na_rep=0)

asim_ao = pd.read_csv(asim_hh_filename)
asim_autos = asim_ao.groupby(["TAZ", "auto_ownership"]).count()["SERIALNO"]
asim_autos = asim_autos.reset_index()
asim_autos = asim_autos.pivot(index="TAZ", columns="auto_ownership")
asim_autos.to_csv("outputs/asim_autos.csv", na_rep=0)

# cdap - ptype count and ptype by M,N,H

if process_tm1:
    tm1_cdap = pd.read_csv(tm1_cdap_filename)
    tm1_cdap_sum = tm1_cdap.groupby(["PersonType", "ActivityString"]).count()["HHID"]
    tm1_cdap_sum = tm1_cdap_sum.reset_index()
    tm1_cdap_sum = tm1_cdap_sum.pivot(index="PersonType", columns="ActivityString")
    tm1_cdap_sum.to_csv("outputs/tm1_cdap.csv", na_rep=0)

asim_cdap = pd.read_csv(asim_per_filename)
asim_cdap_sum = asim_cdap.groupby(["ptype", "cdap_activity"]).count()["household_id"]
asim_cdap_sum = asim_cdap_sum.reset_index()
asim_cdap_sum = asim_cdap_sum.pivot(index="ptype", columns="cdap_activity")
asim_cdap_sum.to_csv("outputs/asim_cdap.csv", na_rep=0)

# free parking by ptype

if process_tm1:
    tm1_per = pd.read_csv(tm1_per_filename)
    tm1_per["fp_choice"] = tm1_per["fp_choice"] == 1  # 1=free, 2==pay
    tm1_work = pd.read_csv(tm1_work_filename)
    tm1_work = tm1_work.set_index("PersonID", drop=False)
    tm1_per["WorkLocation"] = (
        tm1_work["WorkLocation"].loc[tm1_per["person_id"]].tolist()
    )
    tm1_fp = tm1_per[tm1_per["WorkLocation"] > 0]
    tm1_fp = tm1_fp.groupby(["type", "fp_choice"]).count()["hh_id"]
    tm1_fp = tm1_fp.reset_index()
    tm1_fp = tm1_fp.pivot(index="type", columns="fp_choice")
    tm1_fp.to_csv("outputs/tm1_fp.csv", na_rep=0)

asim_cdap["ptypename"] = pd.Series(ptypes)[asim_cdap["ptype"].tolist()].tolist()
asim_fp = asim_cdap.groupby(["ptypename", "free_parking_at_work"]).count()[
    "household_id"
]
asim_fp = asim_fp.reset_index()
asim_fp = asim_fp.pivot(index="ptypename", columns="free_parking_at_work")
asim_fp.to_csv("outputs/asim_fp.csv", na_rep=0)

# value of time

if process_tm1:
    tm1_per = pd.read_csv(tm1_per_filename)
    tm1_per["vot_bin"] = pd.cut(tm1_per["value_of_time"], range(51))
    tm1_per.groupby(["vot_bin"]).count()["hh_id"].to_csv(
        "outputs/tm1_vot.csv", na_rep=0
    )

asim_per = pd.read_csv(asim_per_filename)
asim_per["vot_bin"] = pd.cut(asim_per["value_of_time"], range(51))
asim_per.groupby(["vot_bin"]).count()["household_id"].to_csv(
    "outputs/asim_vot.csv", na_rep=0
)

#############################################################
# TOUR
#############################################################

# indiv mandatory tour freq

tm1_imf_codes = ["", "0", "work1", "work2", "school1", "school2", "work_and_school"]

if process_tm1:
    tm1_per = pd.read_csv(tm1_per_filename)
    tm1_hh = pd.read_csv(tm1_hh_filename)
    tm1_hh = tm1_hh.set_index("hh_id", drop=False)
    tm1_per["hhsize"] = tm1_hh["size"].loc[tm1_per["hh_id"]].tolist()
    # indexing starts at 1
    tm1_per["imf_choice_name"] = pd.Series(tm1_imf_codes)[
        (tm1_per["imf_choice"] + 1).tolist()
    ].tolist()
    tm1_imf = tm1_per.groupby(["type", "imf_choice_name"]).count()["hh_id"]
    tm1_imf = tm1_imf.reset_index()
    tm1_imf = tm1_imf.pivot(index="type", columns="imf_choice_name")

    tm1_imf.to_csv("outputs/tm1_imtf.csv", na_rep=0)

asim_ao = asim_ao.set_index("household_id", drop=False)
asim_cdap["hhsize"] = asim_ao["hhsize"].loc[asim_cdap["household_id"]].tolist()
asim_cdap["ptypename"] = pd.Series(ptypes)[asim_cdap["ptype"].tolist()].tolist()
asim_imf = pd.read_csv(asim_per_filename)
asim_imf["ptypename"] = pd.Series(ptypes)[asim_imf["ptype"].tolist()].tolist()
asim_imf["mandatory_tour_frequency"] = pd.Categorical(
    asim_imf["mandatory_tour_frequency"], categories=tm1_imf_codes
)
asim_imf["mandatory_tour_frequency"][
    asim_imf["mandatory_tour_frequency"].isnull()
] = "0"
asim_imf = asim_imf.groupby(["ptypename", "mandatory_tour_frequency"]).count()[
    "household_id"
]
asim_imf = asim_imf.reset_index()
asim_imf = asim_imf.pivot(index="ptypename", columns="mandatory_tour_frequency")
asim_imf.to_csv("outputs/asim_imtf.csv", na_rep=0)

# indiv mand tour departure and duration

if process_tm1:
    tm1_tours = pd.read_csv(tm1_tour_filename)
    tm1_tours = tm1_tours[tm1_tours["tour_category"] == "MANDATORY"]
    tm1_tours["tour_purpose"][tm1_tours["tour_purpose"].str.contains("work")] = "work"
    tm1_tours["tour_purpose"][tm1_tours["tour_purpose"].str.contains("s")] = "school"
    tm1_mtdd = tm1_tours.groupby(["start_hour", "end_hour", "tour_purpose"]).count()[
        "hh_id"
    ]
    tm1_mtdd = tm1_mtdd.reset_index()

    tm1_mtdd_sch = tm1_mtdd[tm1_mtdd["tour_purpose"] == "school"][
        ["start_hour", "end_hour", "hh_id"]
    ].pivot(index="start_hour", columns="end_hour")
    tm1_mtdd_work = tm1_mtdd[tm1_mtdd["tour_purpose"] == "work"][
        ["start_hour", "end_hour", "hh_id"]
    ].pivot(index="start_hour", columns="end_hour")
    tm1_mtdd_sch.to_csv("outputs/tm1_mtdd_school.csv", na_rep=0)
    tm1_mtdd_work.to_csv("outputs/tm1_mtdd_work.csv", na_rep=0)

asim_tours = pd.read_csv(asim_tour_filename)
asim_tours_man = asim_tours[asim_tours["tour_category"] == "mandatory"]
asim_mtdd = asim_tours_man.groupby(["start", "end", "tour_type"]).count()[
    "household_id"
]
asim_mtdd = asim_mtdd.reset_index()

asim_mtdd_sch = asim_mtdd[asim_mtdd["tour_type"] == "school"][
    ["start", "end", "household_id"]
].pivot(index="start", columns="end")
asim_mtdd_work = asim_mtdd[asim_mtdd["tour_type"] == "work"][
    ["start", "end", "household_id"]
].pivot(index="start", columns="end")

asim_mtdd_sch.to_csv("outputs/asim_mtdd_school.csv", na_rep=0)
asim_mtdd_work.to_csv("outputs/asim_mtdd_work.csv", na_rep=0)

# joint tour frequency

jtf_labels = [
    "",
    "0_tours",
    "1_Shop",
    "1_Main",
    "1_Eat",
    "1_Visit",
    "1_Disc",
    "2_SS",
    "2_SM",
    "2_SE",
    "2_SV",
    "2_SD",
    "2_MM",
    "2_ME",
    "2_MV",
    "2_MD",
    "2_EE",
    "2_EV",
    "2_ED",
    "2_VV",
    "2_VD",
    "2_DD",
]

if process_tm1:
    tm1_jtf = tm1_hh
    tm1_jtf = tm1_jtf[tm1_jtf["jtf_choice"] > 0]
    tm1_jtf["jtf_choice_label"] = pd.Series(jtf_labels)[
        tm1_jtf["jtf_choice"].tolist()
    ].tolist()
    tm1_jtf.groupby("jtf_choice_label").count()["hh_id"].to_csv(
        "outputs/tm1_jtf.csv", na_rep=0
    )

asim_jtf = pd.read_csv(asim_hh_filename)
asim_jtf = asim_jtf[asim_jtf["joint_tour_frequency"] != ""]
asim_jtf.groupby("joint_tour_frequency").count()["household_id"].to_csv(
    "outputs/asim_jtf.csv", na_rep=0
)

# joint tour comp

if process_tm1:
    tm1_jtours = pd.read_csv(tm1_jtour_filename)
    comp_labels = ["", "adult", "children", "mixed"]
    tm1_jtours["tour_composition_labels"] = pd.Series(comp_labels)[
        tm1_jtours["tour_composition"].tolist()
    ].tolist()
    tm1_jtour_comp = tm1_jtours.groupby(
        ["tour_purpose", "tour_composition_labels"]
    ).count()["hh_id"]
    tm1_jtour_comp = tm1_jtour_comp.reset_index()
    tm1_jtour_comp = tm1_jtour_comp.pivot(
        index="tour_purpose", columns="tour_composition_labels"
    )
    tm1_jtour_comp.to_csv("outputs/tm1_jtour_comp.csv", na_rep=0)

asim_jtours = pd.read_csv(asim_tour_filename)
asim_jtours = asim_jtours[asim_jtours["tour_category"] == "joint"]
asim_jtour_comp = asim_jtours.groupby(["tour_type", "composition"]).count()[
    "household_id"
]
asim_jtour_comp = asim_jtour_comp.reset_index()
asim_jtour_comp = asim_jtour_comp.pivot(index="tour_type", columns="composition")
asim_jtour_comp.to_csv("outputs/asim_jtour_comp.csv", na_rep=0)

# joint tour destination

if process_tm1:
    tm1_jtours["distance"] = distmat[
        tm1_jtours["orig_taz"] - 1, tm1_jtours["dest_taz"] - 1
    ]
    tm1_jtours["dist_bin"] = pd.cut(tm1_jtours["distance"], range(51))
    tm1_jtours.groupby(["dist_bin"]).count()["hh_id"].to_csv(
        "outputs/tm1_jtour_dist.csv", na_rep=0
    )

asim_jtours["distance"] = distmat[
    asim_jtours["origin"].astype(int) - 1, asim_jtours["destination"].astype(int) - 1
]
asim_jtours["dist_bin"] = pd.cut(asim_jtours["distance"], range(51))
asim_jtours.groupby(["dist_bin"]).count()["household_id"].to_csv(
    "outputs/asim_jtour_dist.csv", na_rep=0
)

# joint tour tdd

if process_tm1:
    tm1_jtours_tdd = tm1_jtours.groupby(["start_hour", "end_hour"]).count()["hh_id"]
    tm1_jtours_tdd = tm1_jtours_tdd.reset_index()
    tm1_jtours_tdd = tm1_jtours_tdd.pivot(index="start_hour", columns="end_hour")
    tm1_jtours_tdd.to_csv("outputs/tm1_jtours_tdd.csv", na_rep=0)

asim_jtours_tdd = asim_jtours.groupby(["start", "end"]).count()["household_id"]
asim_jtours_tdd = asim_jtours_tdd.reset_index()
asim_jtours_tdd = asim_jtours_tdd.pivot(index="start", columns="end")
asim_jtours_tdd.to_csv("outputs/asim_jtours_tdd.csv", na_rep=0)

# non-mand tour freq

alts = pd.read_csv(asim_nmtf_alts_filename)
alts["ID"] = range(len(alts))

if process_tm1:
    tm1_per = pd.read_csv(tm1_per_filename)
    # 0 doesn't participate in choice model therefore 0 tours, and -1 to align with asim
    tm1_per["inmf_choice"][tm1_per["inmf_choice"] == 0] = 1
    tm1_per["inmf_choice"] = tm1_per["inmf_choice"] - 1
    tm1_nmtf_sum = tm1_per.groupby(["inmf_choice"]).count()["hh_id"]
    tm1_alts = pd.concat([alts, tm1_nmtf_sum], axis=1)
    tm1_alts.to_csv("outputs/tm1_nmtf.csv", na_rep=0)

asim_per_nmtf = pd.read_csv(asim_per_filename)
asim_per_nmtf["ptypename"] = pd.Series(ptypes)[asim_per_nmtf["ptype"].tolist()].tolist()
asim_nmtf_sum = asim_per_nmtf.groupby(["non_mandatory_tour_frequency"]).count()[
    "household_id"
]
asim_alts = pd.concat([alts, asim_nmtf_sum], axis=1)
asim_alts.to_csv("outputs/asim_nmtf.csv", na_rep=0)

# non_mandatory_tour_destination

if process_tm1:
    tm1_tours = pd.read_csv(tm1_tour_filename)
    tm1_tours["distance"] = distmat[
        tm1_tours["orig_taz"] - 1, tm1_tours["dest_taz"] - 1
    ]
    tm1_tours["dist_bin"] = pd.cut(tm1_tours["distance"], range(51))
    tm1_tours_nm = tm1_tours[tm1_tours["tour_category"] == "INDIVIDUAL_NON_MANDATORY"]
    tm1_tours_nm.groupby(["dist_bin"]).count()["hh_id"].to_csv(
        "outputs/tm1_nmtd_dist.csv", na_rep=0
    )

asim_nm_tours = pd.read_csv(asim_tour_filename)
asim_nm_tours = asim_nm_tours[asim_nm_tours["tour_category"] == "non_mandatory"]
asim_nm_tours["distance"] = distmat[
    asim_nm_tours["origin"].astype(int) - 1,
    asim_nm_tours["destination"].astype(int) - 1,
]
asim_nm_tours["dist_bin"] = pd.cut(asim_nm_tours["distance"], range(51))
asim_nm_tours.groupby(["dist_bin"]).count()["household_id"].to_csv(
    "outputs/asim_nmtd_dist.csv", na_rep=0
)

# non_mandatory_tour_scheduling

if process_tm1:
    tm1_nmtours_tdd = tm1_tours_nm.groupby(["start_hour", "end_hour"]).count()["hh_id"]
    tm1_nmtours_tdd = tm1_nmtours_tdd.reset_index()
    tm1_nmtours_tdd = tm1_nmtours_tdd.pivot(index="start_hour", columns="end_hour")
    tm1_nmtours_tdd.to_csv("outputs/tm1_nmtours_tdd.csv", na_rep=0)

asim_nmtours_tdd = asim_nm_tours.groupby(["start", "end"]).count()["household_id"]
asim_nmtours_tdd = asim_nmtours_tdd.reset_index()
asim_nmtours_tdd = asim_nmtours_tdd.pivot(index="start", columns="end")
asim_nmtours_tdd.to_csv("outputs/asim_nmtours_tdd.csv", na_rep=0)

# tour mode choice

if process_tm1:
    tm1_tours = pd.read_csv(tm1_tour_filename)
    tm1_jtours = pd.read_csv(tm1_jtour_filename)
    tm1_tours["tour_mode_labels"] = pd.Series(mode_labels)[
        tm1_tours["tour_mode"].tolist()
    ].tolist()
    tm1_tours["tour_mode_labels"] = pd.Categorical(
        tm1_tours["tour_mode_labels"], categories=mode_labels
    )
    tm1_jtours["tour_mode_labels"] = pd.Series(mode_labels)[
        tm1_jtours["tour_mode"].tolist()
    ].tolist()
    tm1_jtours["tour_mode_labels"] = pd.Categorical(
        tm1_jtours["tour_mode_labels"], categories=mode_labels
    )
    tm1_nmn_tour_mode = tm1_tours.groupby(
        ["tour_mode_labels", "tour_category"]
    ).count()["hh_id"]
    tm1_nmn_tour_mode = tm1_nmn_tour_mode.reset_index()
    tm1_nmn_tour_mode = tm1_nmn_tour_mode.pivot(
        index="tour_mode_labels", columns="tour_category"
    )

    tm1_jtour_mode = tm1_jtours.groupby(["tour_mode_labels", "tour_category"]).count()[
        "hh_id"
    ]
    tm1_jtour_mode = tm1_jtour_mode.reset_index()
    tm1_jtour_mode = tm1_jtour_mode.pivot(
        index="tour_mode_labels", columns="tour_category"
    )

    tm1_tour_mode = pd.concat([tm1_nmn_tour_mode, tm1_jtour_mode], axis=1)
    tm1_tour_mode.columns = ["atwork", "non_mandatory", "mandatory", "joint"]
    tm1_tour_mode = tm1_tour_mode[["atwork", "joint", "mandatory", "non_mandatory"]]
    tm1_tour_mode.to_csv("outputs/tm1_tour_mode.csv", na_rep=0)

asim_tours = pd.read_csv(asim_tour_filename)
asim_tours["tour_mode"] = pd.Categorical(
    asim_tours["tour_mode"], categories=mode_labels
)
asim_tour_mode = asim_tours.groupby(["tour_mode", "tour_category"]).count()[
    "household_id"
]
asim_tour_mode = asim_tour_mode.reset_index()
asim_tour_mode = asim_tour_mode.pivot(index="tour_mode", columns="tour_category")
asim_tour_mode.to_csv("outputs/asim_tour_mode.csv", na_rep=0)

# atwork_subtour_frequency

if process_tm1:
    tm1_work_tours = tm1_tours[tm1_tours["tour_purpose"].str.startswith("work")]
    tm1_atwork_freq_strs = [
        "",
        "no_subtours",
        "eat",
        "business1",
        "maint",
        "business2",
        "eat_business",
    ]
    tm1_work_tours["atWork_freq_str"] = pd.Series(tm1_atwork_freq_strs)[
        tm1_work_tours["atWork_freq"].tolist()
    ].tolist()
    tm1_work_tours.groupby(["atWork_freq_str"]).count()["hh_id"].to_csv(
        "outputs/tm1_atwork_tf.csv", na_rep=0
    )

asim_work_tours = asim_tours[asim_tours["primary_purpose"] == "work"]
asim_work_tours.groupby(["atwork_subtour_frequency"]).count()["household_id"].to_csv(
    "outputs/asim_atwork_tf.csv", na_rep=0
)

# atwork_subtour_destination

if process_tm1:
    tm1_tours = pd.read_csv(tm1_tour_filename)
    tm1_tours["distance"] = distmat[
        tm1_tours["orig_taz"] - 1, tm1_tours["dest_taz"] - 1
    ]
    tm1_tours_atw = tm1_tours[tm1_tours["tour_category"] == "AT_WORK"]
    tm1_tours_atw["dist_bin"] = pd.cut(tm1_tours_atw["distance"], range(51))
    tm1_tours_atw.groupby(["dist_bin"]).count()["hh_id"].to_csv(
        "outputs/tm1_atwork_dist.csv", na_rep=0
    )

asim_atw_tours = pd.read_csv(asim_tour_filename)
asim_atw_tours = asim_atw_tours[asim_atw_tours["tour_category"] == "atwork"]
asim_atw_tours["distance"] = distmat[
    asim_atw_tours["origin"].astype(int) - 1,
    asim_atw_tours["destination"].astype(int) - 1,
]
asim_atw_tours["dist_bin"] = pd.cut(asim_atw_tours["distance"], range(51))
asim_atw_tours.groupby(["dist_bin"]).count()["household_id"].to_csv(
    "outputs/asim_atwork_dist.csv", na_rep=0
)

# atwork_subtour_scheduling

if process_tm1:
    tm1_tours_atw_tdd = tm1_tours_atw.groupby(["start_hour", "end_hour"]).count()[
        "hh_id"
    ]
    tm1_tours_atw_tdd = tm1_tours_atw_tdd.reset_index()
    tm1_tours_atw_tdd = tm1_tours_atw_tdd.pivot(index="start_hour", columns="end_hour")
    tm1_tours_atw_tdd.to_csv("outputs/tm1_atwork_tours_tdd.csv", na_rep=0)

asim_atw_tours_tdd = asim_atw_tours.groupby(["start", "end"]).count()["household_id"]
asim_atw_tours_tdd = asim_atw_tours_tdd.reset_index()
asim_atw_tours_tdd = asim_atw_tours_tdd.pivot(index="start", columns="end")
asim_atw_tours_tdd.to_csv("outputs/asim_atwork_tours_tdd.csv", na_rep=0)

# atwork_subtour_mode_choice - see tour mode above

# tour stop frequency

if process_tm1:
    tm1_tours = pd.read_csv(tm1_tour_filename)
    tm1_jtours = pd.read_csv(tm1_jtour_filename)

    tm1_tours["tour_purpose_simple"] = tm1_tours["tour_purpose"]
    tm1_tours["tour_purpose_simple"] = tm1_tours["tour_purpose_simple"].str.replace(
        "atwork_", ""
    )
    tm1_tours["tour_purpose_simple"][
        tm1_tours["tour_purpose_simple"].str.contains("work_")
    ] = "work"
    tm1_tours["tour_purpose_simple"][
        tm1_tours["tour_purpose_simple"].str.contains("school_")
    ] = "school"
    tm1_tours["tour_purpose_simple"][
        tm1_tours["tour_purpose_simple"].str.contains("university")
    ] = "school"
    tm1_tours["tour_purpose_simple"][
        tm1_tours["tour_purpose_simple"].str.contains("escort_")
    ] = "escort"
    tm1_tours_atw = tm1_tours[tm1_tours["tour_category"] == "AT_WORK"]
    tm1_tours_nmn = tm1_tours[tm1_tours["tour_category"] != "AT_WORK"]

    tm1_tours_nmn["tsf"] = (
        tm1_tours_nmn["num_ob_stops"].astype(str)
        + "-"
        + tm1_tours_nmn["num_ib_stops"].astype(str)
    )
    tm1_stop_freq = tm1_tours_nmn.groupby(["tsf", "tour_purpose_simple"]).count()[
        "hh_id"
    ]
    tm1_stop_freq = tm1_stop_freq.reset_index()
    tm1_stop_freq = tm1_stop_freq.pivot(index="tsf", columns="tour_purpose_simple")

    tm1_jtours["tsf"] = (
        tm1_jtours["num_ob_stops"].astype(str)
        + "-"
        + tm1_jtours["num_ib_stops"].astype(str)
    )
    tm1_tours_atw["tsf"] = (
        tm1_tours_atw["num_ob_stops"].astype(str)
        + "-"
        + tm1_tours_atw["num_ib_stops"].astype(str)
    )

    tm1_stop_freq_joint = tm1_jtours.groupby(["tsf"]).count()["hh_id"]
    tm1_stop_freq_atwork = tm1_tours_atw.groupby(["tsf"]).count()["hh_id"]
    tm1_stop_freq = pd.concat(
        [tm1_stop_freq, tm1_stop_freq_joint, tm1_stop_freq_atwork], axis=1
    )
    tm1_stop_freq.to_csv("outputs/tm1_stop_freq.csv", na_rep=0)

asim_tours = pd.read_csv(asim_tour_filename)
asim_nmn_tours = asim_tours[
    (asim_tours["tour_category"] == "mandatory")
    | (asim_tours["tour_category"] == "non_mandatory")
]
asim_joint_tours = asim_tours[asim_tours["tour_category"] == "joint"]
asim_atw_tours = asim_tours[asim_tours["tour_category"] == "atwork"]
asim_stop_freq = asim_nmn_tours.groupby(["stop_frequency", "tour_type"]).count()[
    "household_id"
]
asim_stop_freq = asim_stop_freq.reset_index()
asim_stop_freq = asim_stop_freq.pivot(index="stop_frequency", columns="tour_type")

asim_stop_freq_joint = asim_joint_tours.groupby(["stop_frequency"]).count()[
    "household_id"
]
asim_stop_freq_atwork = asim_atw_tours.groupby(["stop_frequency"]).count()[
    "household_id"
]
asim_stop_freq = pd.concat(
    [asim_stop_freq, asim_stop_freq_joint, asim_stop_freq_atwork], axis=1
)
asim_stop_freq.to_csv("outputs/asim_stop_freq.csv", na_rep=0)

#############################################################
# TRIP
#############################################################

# trip purpose

if process_tm1:
    tm1_trips = pd.read_csv(tm1_trips_filename)
    tm1_jtrips = pd.read_csv(tm1_jtrips_filename)
    tm1_trips["orig_purpose"][tm1_trips["orig_purpose"] == "university"] = "univ"
    tm1_trips["orig_purpose"] = pd.Categorical(tm1_trips["orig_purpose"])
    tm1_jtrips["orig_purpose"] = pd.Categorical(
        tm1_jtrips["orig_purpose"], categories=tm1_trips["orig_purpose"].cat.categories
    )
    tm1_trip_purp = tm1_trips.groupby(["orig_purpose", "tour_category"]).count()[
        "hh_id"
    ]
    tm1_trip_purp = tm1_trip_purp.reset_index()
    tm1_trip_purp = tm1_trip_purp.pivot(index="orig_purpose", columns="tour_category")
    tm1_jtrip_purp = tm1_jtrips.groupby(["orig_purpose"]).count()["hh_id"]
    tm1_trip_purp = pd.concat([tm1_trip_purp, tm1_jtrip_purp], axis=1)
    tm1_trip_purp.columns = ["atwork", "non_mandatory", "mandatory", "joint"]
    tm1_trip_purp = tm1_trip_purp[["atwork", "joint", "mandatory", "non_mandatory"]]
    tm1_trip_purp.to_csv("outputs/tm1_trip_purp.csv", na_rep=0)

asim_trips = pd.read_csv(asim_trips_filename)
asim_tours = pd.read_csv(asim_tour_filename)
asim_tours = asim_tours.set_index("tour_id", drop=False)
asim_trips["tour_category"] = (
    asim_tours["tour_category"].loc[asim_trips["tour_id"]].tolist()
)
asim_trip_purp = asim_trips.groupby(["purpose", "tour_category"]).count()[
    "household_id"
]
asim_trip_purp = asim_trip_purp.reset_index()
asim_trip_purp = asim_trip_purp.pivot(index="purpose", columns="tour_category")
asim_trip_purp.to_csv("outputs/asim_trip_purp.csv", na_rep=0)

# trip destination

if process_tm1:
    tm1_trips["distance"] = distmat[
        tm1_trips["orig_taz"] - 1, tm1_trips["dest_taz"] - 1
    ]
    tm1_jtrips["distance"] = distmat[
        tm1_jtrips["orig_taz"] - 1, tm1_jtrips["dest_taz"] - 1
    ]
    tm1_trips["dist_bin"] = pd.cut(tm1_trips["distance"], range(51))
    tm1_jtrips["dist_bin"] = pd.cut(tm1_jtrips["distance"], range(51))
    tm1_trips_dist = pd.concat(
        [
            tm1_trips.groupby(["dist_bin"]).count()["hh_id"]
            + tm1_jtrips.groupby(["dist_bin"]).count()["hh_id"]
        ],
        axis=1,
    )
    tm1_trips_dist.to_csv("outputs/tm1_trips_dist.csv", na_rep=0)

asim_trips["distance"] = distmat[
    asim_trips["origin"] - 1, asim_trips["destination"] - 1
]
asim_trips["dist_bin"] = pd.cut(asim_trips["distance"], range(51))
asim_trips.groupby(["dist_bin"]).count()["household_id"].to_csv(
    "outputs/asim_trips_dist.csv", na_rep=0
)

# trip scheduling

if process_tm1:
    tm1_trips_tdd = (
        tm1_trips.groupby(["depart_hour"]).count()["hh_id"]
        + tm1_jtrips.groupby(["depart_hour"]).count()["hh_id"]
    )
    tm1_trips_tdd.to_csv("outputs/tm1_trips_depart.csv", na_rep=0)

asim_trips_tdd = asim_trips.groupby(["depart"]).count()["household_id"]
asim_trips_tdd.to_csv("outputs/asim_trips_depart.csv", na_rep=0)

# trip mode share by tour purpose

if process_tm1:
    tm1_trips["trip_mode_str"] = pd.Series(mode_labels)[
        tm1_trips["trip_mode"].tolist()
    ].tolist()
    tm1_trips["trip_mode_str"] = pd.Categorical(
        tm1_trips["trip_mode_str"], categories=mode_labels
    )
    tm1_jtrips["trip_mode_str"] = pd.Series(mode_labels)[
        tm1_jtrips["trip_mode"].tolist()
    ].tolist()
    tm1_jtrips["trip_mode_str"] = pd.Categorical(
        tm1_jtrips["trip_mode_str"], categories=mode_labels
    )

    tm1_trip_mode = tm1_trips.groupby(["trip_mode_str", "tour_category"]).count()[
        "hh_id"
    ]
    tm1_trip_mode = tm1_trip_mode.reset_index()
    tm1_trip_mode = tm1_trip_mode.pivot(index="trip_mode_str", columns="tour_category")

    tm1_jtrip_mode = tm1_jtrips.groupby(["trip_mode_str"]).count()["hh_id"]
    tm1_trip_mode = pd.concat([tm1_trip_mode, tm1_jtrip_mode], axis=1)
    tm1_trip_mode.columns = ["atwork", "non_mandatory", "mandatory", "joint"]
    tm1_trip_mode = tm1_trip_mode[["atwork", "joint", "mandatory", "non_mandatory"]]
    tm1_trip_mode.to_csv("outputs/tm1_trip_mode.csv", na_rep=0)

asim_trips["trip_mode"] = pd.Categorical(
    asim_trips["trip_mode"], categories=mode_labels
)
asim_trip_mode = asim_trips.groupby(["trip_mode", "tour_category"]).count()[
    "household_id"
]
asim_trip_mode = asim_trip_mode.reset_index()
asim_trip_mode = asim_trip_mode.pivot(index="trip_mode", columns="tour_category")
asim_trip_mode.to_csv("outputs/asim_trip_mode.csv", na_rep=0)



# ActivitySim
# Copyright (C) 2016 RSG Inc
# See full license in LICENSE.txt.
# run from the mtc tm1 skims folder

import os

import openmatrix as omx
import pandas as pd


def read_manifest(manifest_file_name):

    column_map = {
        "Token": "skim_key1",
        "TimePeriod": "skim_key2",
        "File": "source_file_name",
        "Matrix": "source_key",
    }
    converters = {col: str for col in column_map.keys()}

    manifest = pd.read_csv(
        manifest_file_name, header=0, comment="#", converters=converters
    )

    manifest.rename(columns=column_map, inplace=True)

    return manifest


def omx_getMatrix(omx_file_name, omx_key):

    with omx.open_file(omx_file_name, "r") as omx_file:

        if omx_key not in omx_file.list_matrices():
            print(
                "Source matrix with key '%s' not found in file '%s"
                % (
                    omx_key,
                    omx_file,
                )
            )
            print(omx_file.list_matrices())
            raise RuntimeError(
                "Source matrix with key '%s' not found in file '%s"
                % (
                    omx_key,
                    omx_file,
                )
            )

        data = omx_file[omx_key]

    return data


manifest_dir = "."
source_data_dir = "."
dest_data_dir = "."

manifest_file_name = os.path.join(manifest_dir, "skim_manifest.csv")
dest_file_name = os.path.join(dest_data_dir, "skims.omx")

with omx.open_file(dest_file_name, "a") as dest_omx:

    manifest = read_manifest(manifest_file_name)

    for row in manifest.itertuples(index=True):

        source_file_name = os.path.join(source_data_dir, row.source_file_name)

        if row.skim_key2:
            dest_key = row.skim_key1 + "__" + row.skim_key2
        else:
            dest_key = row.skim_key1

        print(
            "Reading '%s' from '%s' in %s"
            % (
                dest_key,
                row.source_key,
                source_file_name,
            )
        )
        with omx.open_file(source_file_name, "r") as source_omx:

            if row.source_key not in source_omx.list_matrices():
                print(
                    "Source matrix with key '%s' not found in file '%s"
                    % (
                        row.source_key,
                        source_file_name,
                    )
                )
                print(source_omx.list_matrices())
                raise RuntimeError(
                    "Source matrix with key '%s' not found in file '%s"
                    % (
                        row.source_key,
                        dest_omx,
                    )
                )

            data = source_omx[row.source_key]

            if dest_key in dest_omx.list_matrices():
                print("deleting existing dest key '%s'" % (dest_key,))
                dest_omx.removeNode(dest_omx.root.data, dest_key)

            dest_omx[dest_key] = data



# ActivitySim
# See full license in LICENSE.txt.

import logging
import sys

from activitysim.core import chunk, config, inject, mem, mp_tasks, pipeline, tracing

# from activitysim import abm


logger = logging.getLogger("activitysim")


def cleanup_output_files():

    active_log_files = [
        h.baseFilename
        for h in logger.root.handlers
        if isinstance(h, logging.FileHandler)
    ]
    tracing.delete_output_files("log", ignore=active_log_files)

    tracing.delete_output_files("h5")
    tracing.delete_output_files("csv")
    tracing.delete_output_files("txt")
    tracing.delete_output_files("yaml")
    tracing.delete_output_files("prof")
    tracing.delete_output_files("omx")


def run(run_list, injectables=None):

    if run_list["multiprocess"]:
        logger.info("run multiprocess simulation")
        mp_tasks.run_multiprocess(run_list, injectables)
    else:
        logger.info("run single process simulation")
        pipeline.run(models=run_list["models"], resume_after=run_list["resume_after"])
        pipeline.close_pipeline()
        mem.log_global_hwm()


def log_settings(injectables):

    settings = [
        "households_sample_size",
        "chunk_size",
        "multiprocess",
        "num_processes",
        "resume_after",
    ]

    for k in settings:
        logger.info("setting %s: %s" % (k, config.setting(k)))

    for k in injectables:
        logger.info("injectable %s: %s" % (k, inject.get_injectable(k)))


if __name__ == "__main__":

    inject.add_injectable("data_dir", "data")
    inject.add_injectable("configs_dir", "configs")

    config.handle_standard_args()

    config.filter_warnings()
    tracing.config_logger()

    # log_settings(injectables)

    t0 = tracing.print_elapsed_time()

    # cleanup if not resuming
    if not config.setting("resume_after", False):
        cleanup_output_files()

    run_list = mp_tasks.get_run_list()

    if run_list["multiprocess"]:
        # do this after config.handle_standard_args, as command line args may override injectables
        injectables = list(
            set(injectables) | set(["data_dir", "configs_dir", "output_dir"])
        )
        injectables = {k: inject.get_injectable(k) for k in injectables}
    else:
        injectables = None

    run(run_list, injectables)

    # pipeline will be close if multiprocessing
    # if you want access to tables, BE SURE TO OPEN WITH '_' or all tables will be reinitialized
    # pipeline.open_pipeline('_')

    t0 = tracing.print_elapsed_time("everything", t0)



# rst doesn't support image maps, so we'll add one after the html has been built

fileName = "_build/html/examples.html"
line = 'src="_images/abmexample.jpg" />'
lineWithMap = 'src="_images/abmexample.jpg" usemap="#image-map" />'
imageMap = """\n   <map name="image-map">
    <area target="" alt="" title="" href="models.html#write-trip-matrices" coords="234,683,436,709" shape="rect">
    <area target="" alt="" title="" href="models.html#trip-mode-choice" coords="438,670,235,646" shape="rect">
    <area target="" alt="" title="" href="models.html#trip-scheduling" coords="235,607,434,630" shape="rect">
    <area target="" alt="" title="" href="models.html#trip-destination-choice" coords="235,565,433,588" shape="rect">
    <area target="" alt="" title="" href="models.html#trip-purpose" coords="236,524,436,550" shape="rect">
    <area target="" alt="" title="" href="models.html#intermediate-stop-frequency" coords="5,481,663,505" shape="rect">
    <area target="" alt="" title="" href="models.html#atwork-subtour-scheduling" coords="516,401,666,427" shape="rect">
    <area target="" alt="" title="" href="models.html#atwork-subtour-destination" coords="517,361,667,385" shape="rect">
    <area target="" alt="" title="" href="models.html#atwork-subtour-frequency" coords="515,243,667,265" shape="rect">
    <area target="" alt="" title="" href="models.html#non-mandatory-tour-scheduling" coords="347,404,497,425" shape="rect">
    <area target="" alt="" title="" href="models.html#non-mandatory-tour-destination-choice" coords="347,363,496,384" shape="rect">
    <area target="" alt="" title="" href="models.html#non-mandatory-tour-frequency" coords="347,245,495,267" shape="rect">
    <area target="" alt="" title="" href="models.html#joint-tour-scheduling" coords="175,402,327,424" shape="rect">
    <area target="" alt="" title="" href="models.html#joint-tour-destination-choice" coords="175,361,326,386" shape="rect">
    <area target="" alt="" title="" href="models.html#joint-tour-participation" coords="175,324,326,345" shape="rect">
    <area target="" alt="" title="" href="models.html#joint-tour-composition" coords="176,282,326,307" shape="rect">
    <area target="" alt="" title="" href="models.html#joint-tour-frequency" coords="176,243,327,266" shape="rect">
    <area target="" alt="" title="" href="models.html#tour-mode" coords="5,442,667,464" shape="rect">
    <area target="" alt="" title="" href="models.html#mandatory-tour-scheduling" coords="5,401,156,425" shape="rect">
    <area target="" alt="" title="" href="models.html#mandatory-tour-frequency" coords="5,241,157,267" shape="rect">
    <area target="" alt="" title="" href="models.html#cdap" coords="235,165,434,189" shape="rect">
    <area target="" alt="" title="" href="models.html#free-parking-eligibility" coords="239,126,435,148" shape="rect">
    <area target="" alt="" title="" href="models.html#auto-ownership" coords="237,86,433,109" shape="rect">
    <area target="" alt="" title="" href="models.html#school-location" coords="237,43,435,67" shape="rect">
    <area target="" alt="" title="" href="models.html#accessibility" coords="236,6,436,27" shape="rect">
    <area target="" alt="" title="" href="models.html#work-from-home" coords="500,25,675,53" shape="rect">
    <area target="" alt="" title="" href="models.html#transit-pass-ownership" coords="501,67,676,95" shape="rect">
    <area target="" alt="" title="" href="models.html#telecommute-frequency" coords="502,108,677,136" shape="rect">
   </map>
    """  # noqa

print("add image map to " + fileName)

with open(fileName, encoding="utf-8") as file:
    lines = file.readlines()

with open(fileName, "w") as file:
    for aLine in lines:
        if line in aLine:
            print("updated " + fileName)
            file.writelines("%s" % aLine.replace(line, lineWithMap))
            file.writelines("%s" % imageMap)
        else:
            file.writelines("%s" % aLine)



# create table of pipeline table fields by creator
# Ben Stabler, ben.stabler@rsginc.com, 06/06/18
# C:\projects\development\activitysim\example>python ../scripts/make_pipeline_output.py

import pandas as pd

pipeline_filename = "output\\pipeline.h5"
out_fields_filename = "output\\pipeline_fields.csv"

# get pipeline tables
pipeline = pd.io.pytables.HDFStore(pipeline_filename)
checkpoints = pipeline["/checkpoints"]
p_tables = pipeline.keys()
p_tables.remove("/checkpoints")
p_tables_tables = [i.split("/")[1] for i in p_tables]
p_tables_cps = [i.split("/")[2] for i in p_tables]
p_tables = pd.DataFrame({"table": p_tables_tables, "cp": p_tables_cps})

# join timestamps and sort
cp_times = checkpoints[["checkpoint_name", "timestamp"]]
cp_times = cp_times.set_index("checkpoint_name")
p_tables = p_tables.join(cp_times, on="cp")
p_tables = p_tables.sort_values(by=["table", "timestamp"])

# build table of fields for each table by creator
# TABLE, FIELD, DTYPE, CREATOR, NCOL, NROW
tables_fields = dict()
for i in range(len(p_tables)):

    cur_table = p_tables["table"].iloc[i]
    cur_cp = p_tables["cp"].iloc[i]

    print("process " + cur_table + "/" + cur_cp)

    cur_table_data = pipeline["/" + cur_table + "/" + cur_cp]
    cur_table_data_fields = cur_table_data.dtypes.index
    cur_table_data_dtypes = cur_table_data.dtypes.values
    cur_table_data_nrow = len(cur_table_data)
    cur_table_data_ncol = len(cur_table_data.columns)

    table_data = pd.DataFrame(
        {
            "TABLE": cur_table,
            "FIELD": cur_table_data_fields,
            "DTYPE": cur_table_data_dtypes,
            "CREATOR": cur_cp,
            "NCOL": cur_table_data_ncol,
            "NROW": cur_table_data_nrow,
        }
    )
    table_data = table_data[
        ["TABLE", "FIELD", "DTYPE", "CREATOR", "NCOL", "NROW"]
    ]  # reorder

    if cur_table not in tables_fields.keys():
        tables_fields[cur_table] = table_data
    else:
        # determine new fields and append to table
        existing_fields = tables_fields[cur_table].FIELD
        new_fields = table_data[~table_data.FIELD.isin(existing_fields)]
        tables_fields[cur_table] = tables_fields[cur_table].append(new_fields)

# write final table
table_names = sorted(tables_fields.keys())
for i in range(len(table_names)):
    if i == 0:
        final_fields = tables_fields[table_names[i]]
    else:
        final_fields = final_fields.append(tables_fields[table_names[i]])
final_fields.to_csv(out_fields_filename)



from typing import Union

try:
    from pydantic import BaseModel as PydanticBase
except ModuleNotFoundError:

    class PydanticBase:
        pass


class InputTable(PydanticBase):
    """
    The features that define an input table to be read by ActivitySim.
    """

    tablename: str
    """Name of the injected table"""

    filename: str = None
    """
    Name of the CSV or HDF5 file to read.

    If not provided, defaults to `input_store`
    """

    index_col: str = None
    """table column to use for the index"""

    rename_columns: dict[str, str] = None
    """dictionary of column name mappings"""

    keep_columns: list[str] = None
    """
    columns to keep once read in to memory.

    Save only the columns needed for modeling or analysis to save on memory
    and file I/O
    """

    h5_tablename: str = None
    """table name if reading from HDF5 and different from `tablename`"""


class Settings(PydanticBase):
    """
    The overall settings for the ActivitySim model system.

    The input for these settings is typically stored in one main YAML file,
    usually called ``settings.yaml``.

    Note that this implementation is presently used only for generating
    documentation, but future work may migrate the settings implementation to
    actually use this pydantic code to validate the settings before running
    the model.
    """

    models: list[str]
    """
    list of model steps to run - auto ownership, tour frequency, etc.

    See :ref:`model_steps` for more details about each step.
    """

    resume_after: str = None
    """to resume running the data pipeline after the last successful checkpoint"""

    input_table_list: list[InputTable]
    """list of table names, indices, and column re-maps for each table in `input_store`"""

    input_store: str = None
    """HDF5 inputs file"""

    create_input_store: bool = False
    """
    Write the inputs as read in back to an HDF5 store.

    If enabled, this writes the store to the outputs folder to use for subsequent
    model runs, as reading HDF5 can be faster than reading CSV files."""

    households_sample_size: int = None
    """
    Number of households to sample and simulate

    If omitted or set to 0, ActivitySim will simulate all households.
    """
    trace_hh_id: Union[int, list] = None
    """
    Trace household id(s)

    If omitted, no tracing is written out
    """

    trace_od: list[int] = None
    """
    Trace origin, destination pair in accessibility calculation

    If omitted, no tracing is written out.
    """

    chunk_training_mode: str = None
    """
    The method to use for chunk training.

    Valid values include {disabled, training, production, adaptive}.
    See :ref:`chunk_size` for more details.
    """

    chunk_size: int = None
    """
    Approximate amount of RAM to allocate to ActivitySim for batch processing.

    See :ref:`chunk_size` for more details.
    """

    chunk_method: str = None
    """
    Memory use measure to use for chunking.

    See :ref:`chunk_size`.
    """

    checkpoints: Union[bool, list] = True
    """
    When to write checkpoint (intermediate table states) to disk.

    If True, checkpoints are written at each step. If False, no intermediate
    checkpoints will be written before the end of run.  Or, provide an explicit
    list of models to checkpoint.
    """

    check_for_variability: bool = False
    """
    Debugging feature to find broken model specifications.

    Enabling this check does not alter valid results but slows down model runs.
    """

    log_alt_losers: bool = False
    """
    Write out expressions when all alternatives are unavailable.

    This can be useful for model development to catch errors in specifications.
    Enabling this check does not alter valid results but slows down model runs.
    """

    use_shadow_pricing: bool = False
    """turn shadow_pricing on and off for work and school location"""

    output_tables: list[str] = None
    """list of output tables to write to CSV or HDF5"""

    want_dest_choice_sample_tables: bool = False
    """turn writing of sample_tables on and off for all models"""

    cleanup_pipeline_after_run: bool = False
    """
    Cleans up pipeline after successful run.

    This will clean up pipeline only after successful runs, by creating a
    single-checkpoint pipeline file, and deleting any subprocess pipelines.
    """

    sharrow: Union[bool, str] = False
    """
    Set the sharrow operating mode.

    .. versionadded:: 1.2

    * `false` - Do not use sharrow.  This is the default if no value is given.
    * `true` - Use sharrow optimizations when possible, but fall back to
      legacy `pandas.eval` systems when any error is encountered.  This is the
      preferred mode for running with sharrow if reliability is more important
      than performance.
    * `require` - Use sharrow optimizations, and raise an error if they fail
      unexpectedly.  This is the preferred mode for running with sharrow
      if performance is a concern.
    * `test` - Run every relevant calculation using both sharrow and legacy
      systems, and compare them to ensure the results match.  This is the slowest
      mode of operation, but useful for development and debugging.
    """


class ZarrDigitalEncoding(PydanticBase):
    """Digital encoding instructions for skim tables.

    .. versionadded:: 1.2
    """

    regex: str
    """A regular expression for matching skim matrix names.

    All skims with names that match under typical regular expression rules
    for Python will be processed together.
    """

    joint_dict: str
    """The name of the joint dictionary for this group.

    This must be a unique name for this set of skims, and a new array
    will be added to the Dataset with this name.  It will be an integer-
    type array indicating the position of each element in the jointly
    encoded dictionary."""


class TAZ_Settings(PydanticBase):
    """
    Complex settings for TAZ skims that are not just OMX file(s).

    .. versionadded:: 1.2
    """

    omx: str = None
    """The filename of the data stored in OMX format.

    This is treated as a fallback for the raw input data, if ZARR format data
    is not available.
    """

    zarr: str = None
    """The filename of the data stored in ZARR format.

    Reading ZARR data can be much faster than reading OMX format data, so if
    this filename is given, the ZARR file format is preferred if it exists. If
    it does not exist, then OMX data is read in and then ZARR data is written
    out for future usage.

    .. versionadded:: 1.2
    """

    zarr_digital_encoding: list[ZarrDigitalEncoding] = None
    """
    A list of encodings to apply before saving skims in ZARR format.

    .. versionadded:: 1.2
    """


class NetworkSettings(PydanticBase):
    """
    Network level of service and skims settings

    The input for these settings is typically stored in one YAML file,
    usually called ``network_los.yaml``.
    """

    zone_system: int
    """Which zone system type is used.

    * 1 - TAZ only.
    * 2 - MAZ and TAZ.
    * 3 - MAZ, TAZ, and TAP
    """

    taz_skims: Union[str, TAZ_Settings] = None
    """Instructions for how to load and pre-process skim matrices.

    If given as a string, it is interpreted as the location for OMX file(s),
    either as a single file or as a glob-matching pattern for multiple files.
    The time period for the matrix must be represented at the end of the matrix
    name and be seperated by a double_underscore (e.g. `BUS_IVT__AM` indicates base
    skim BUS_IVT with a time period of AM.

    Alternatively, this can be given as a nested dictionary defined via the
    TAZ_Settings class, which allows for ZARR transformation and pre-processing.
    """

    skim_time_periods: dict
    """time period upper bound values and labels

    * ``time_window`` - total duration (in minutes) of the modeled time span (Default: 1440 minutes (24 hours))
    * ``period_minutes`` - length of time (in minutes) each model time period represents. Must be whole factor of ``time_window``. (Default: 60 minutes)
    * ``periods`` - Breakpoints that define the aggregate periods for skims and assignment
    * ``labels`` - Labels to define names for aggregate periods for skims and assignment
    """

    read_skim_cache: bool = False
    """Read cached skims (using numpy memmap) from output directory.

    Reading from memmap is much faster than omx, but the memmap is a huge
    uncompressed file.
    """

    write_skim_cache: bool = False
    """Write memmapped cached skims to output directory.

    This is needed if you want to use the cached skims to speed up subsequent
    runs.
    """

    cache_dir: str = None
    """alternate dir to read/write cache files (defaults to output_dir)"""



# ActivitySim
# See full license in LICENSE.txt.
import logging

import numpy as np

from activitysim.core import tracing
from activitysim.core import config
from activitysim.core import pipeline
from activitysim.core import simulate
from activitysim.core import inject
from activitysim.core import expressions

from activitysim.abm.models.util import estimation

logger = logging.getLogger(__name__)


@inject.step()
def work_from_home(persons_merged, persons, chunk_size, trace_hh_id):
    """
    This model predicts the whether a person (worker) works from home. The output
    from this model is TRUE (if works from home) or FALSE (works away from home).
    The workplace location choice is overridden for workers who work from home
    and set to -1.

    The main interface to the work from home model is the work_from_home() function.
    This function is registered as an orca step in the example Pipeline.
    """

    trace_label = "work_from_home"
    model_settings_file_name = "work_from_home.yaml"

    choosers = persons_merged.to_frame()
    choosers = choosers[choosers.workplace_zone_id > -1]
    logger.info("Running %s with %d persons", trace_label, len(choosers))

    model_settings = config.read_model_settings(model_settings_file_name)
    estimator = estimation.manager.begin_estimation("work_from_home")

    constants = config.get_model_constants(model_settings)

    # - preprocessor
    preprocessor_settings = model_settings.get("preprocessor", None)
    if preprocessor_settings:

        locals_d = {}
        if constants is not None:
            locals_d.update(constants)

        expressions.assign_columns(
            df=choosers,
            model_settings=preprocessor_settings,
            locals_dict=locals_d,
            trace_label=trace_label,
        )

    model_spec = simulate.read_model_spec(file_name=model_settings["SPEC"])
    coefficients_df = simulate.read_model_coefficients(model_settings)
    model_spec = simulate.eval_coefficients(model_spec, coefficients_df, estimator)

    nest_spec = config.get_logit_model_settings(model_settings)

    if estimator:
        estimator.write_model_settings(model_settings, model_settings_file_name)
        estimator.write_spec(model_settings)
        estimator.write_coefficients(coefficients_df)
        estimator.write_choosers(choosers)

    choices = simulate.simple_simulate(
        choosers=choosers,
        spec=model_spec,
        nest_spec=nest_spec,
        locals_d=constants,
        chunk_size=chunk_size,
        trace_label=trace_label,
        trace_choice_name="work_from_home",
        estimator=estimator,
    )

    work_from_home_alt = model_settings["WORK_FROM_HOME_ALT"]
    choices = choices == work_from_home_alt

    dest_choice_column_name = model_settings["DEST_CHOICE_COLUMN_NAME"]
    print(dest_choice_column_name)

    if estimator:
        estimator.write_choices(choices)
        choices = estimator.get_survey_values(choices, "persons", "work_from_home")
        estimator.write_override_choices(choices)
        estimator.end_estimation()

    persons = persons.to_frame()
    persons["work_from_home"] = choices.reindex(persons.index).fillna(0).astype(bool)
    persons[dest_choice_column_name] = np.where(
        persons.work_from_home is True, -1, persons[dest_choice_column_name]
    )

    pipeline.replace_table("persons", persons)

    tracing.print_summary("work_from_home", persons.work_from_home, value_counts=True)

    if trace_hh_id:
        tracing.trace_df(persons, label=trace_label, warn_if_empty=True)



import argparse
import os

import numpy as np
import openmatrix as omx
import pandas as pd

parser = argparse.ArgumentParser(description="crop PSRC raw_data")
parser.add_argument(
    "input", metavar="input_file_name", type=str, nargs=1, help=f"input omx file"
)

parser.add_argument(
    "output", metavar="output_file_name", type=str, nargs=1, help=f"output omx file"
)

args = parser.parse_args()


input_file_name = args.input[0]
output_file_name = args.output[0]


#
# skims
#
skim_data_type = np.float32

omx_in = omx.open_file(input_file_name, "r")
print(f"omx_in shape {omx_in.shape()}")

omx_out = omx.open_file(output_file_name, "w")

for mapping_name in omx_in.listMappings():
    offset_map = omx_in.mapentries(mapping_name)
    omx_out.create_mapping(mapping_name, offset_map)


for mat_name in omx_in.list_matrices():

    # make sure we have a vanilla numpy array, not a CArray
    m = np.asanyarray(omx_in[mat_name])
    print(f"{mat_name} {m.shape} {type(m[0,0])}")

    omx_out[mat_name] = m.astype(skim_data_type)


omx_in.close()
omx_out.close()
